@@ -6,9 +6,11 @@ use std::slice::Iter;
 
 use crate::coord_units::CoordUnits;
 use crate::document::{AcquiredNodes, NodeId};
-use crate::drawing_ctx::ViewParams;
+use crate::drawing_ctx::DrawingCtx;
 use crate::element::{Draw, Element, ElementResult, SetAttributes};
 use crate::error::ValueErrorKind;
+use crate::filter_func::FilterFunction;
+use crate::filters::{extract_filter_from_filter_node, FilterResolveError, FilterSpec};
 use crate::length::*;
 use crate::node::NodeBorrow;
 use crate::parsers::{Parse, ParseValue};
@@ -27,7 +29,7 @@ pub struct Filter {
 }
 
 /// A <filter> element definition in user-space coordinates.
-pub struct ResolvedFilter {
+pub struct UserSpaceFilter {
     pub rect: Rect,
     pub filter_units: CoordUnits,
     pub primitive_units: CoordUnits,
@@ -56,15 +58,15 @@ impl Filter {
         self.primitive_units
     }
 
-    pub fn resolve(&self, values: &ComputedValues, params: &ViewParams) -> ResolvedFilter {
-        let x = self.x.normalize(values, &params);
-        let y = self.y.normalize(values, &params);
-        let w = self.width.normalize(values, &params);
-        let h = self.height.normalize(values, &params);
+    pub fn to_user_space(&self, params: &NormalizeParams) -> UserSpaceFilter {
+        let x = self.x.to_user(params);
+        let y = self.y.to_user(params);
+        let w = self.width.to_user(params);
+        let h = self.height.to_user(params);
 
         let rect = Rect::new(x, y, x + w, y + h);
 
-        ResolvedFilter {
+        UserSpaceFilter {
             rect,
             filter_units: self.filter_units,
             primitive_units: self.primitive_units,
@@ -104,7 +106,75 @@ impl Draw for Filter {}
 #[derive(Debug, Clone, PartialEq)]
 pub enum FilterValue {
     Url(NodeId),
-    // TODO: add functions from https://www.w3.org/TR/filter-effects-1/#filter-functions
+    Function(FilterFunction),
+}
+
+impl FilterValue {
+    pub fn to_filter_spec(
+        &self,
+        acquired_nodes: &mut AcquiredNodes<'_>,
+        values: &ComputedValues,
+        draw_ctx: &DrawingCtx,
+        node_being_filtered_name: &str,
+    ) -> Result<FilterSpec, FilterResolveError> {
+        match *self {
+            FilterValue::Url(ref node_id) => filter_spec_from_filter_node(
+                acquired_nodes,
+                draw_ctx,
+                node_id,
+                node_being_filtered_name,
+            ),
+
+            FilterValue::Function(ref func) => func.to_filter_spec(values, draw_ctx),
+        }
+    }
+}
+
+fn filter_spec_from_filter_node(
+    acquired_nodes: &mut AcquiredNodes<'_>,
+    draw_ctx: &DrawingCtx,
+    node_id: &NodeId,
+    node_being_filtered_name: &str,
+) -> Result<FilterSpec, FilterResolveError> {
+    acquired_nodes
+        .acquire(node_id)
+        .map_err(|e| {
+            rsvg_log!(
+                "element {} will not be filtered with \"{}\": {}",
+                node_being_filtered_name,
+                node_id,
+                e
+            );
+            FilterResolveError::ReferenceToNonFilterElement
+        })
+        .and_then(|acquired| {
+            let node = acquired.get();
+            let element = node.borrow_element();
+
+            match *element {
+                Element::Filter(_) => {
+                    if element.is_in_error() {
+                        rsvg_log!(
+                            "element {} will not be filtered since its filter \"{}\" is in error",
+                            node_being_filtered_name,
+                            node_id,
+                        );
+                        Err(FilterResolveError::ChildNodeInError)
+                    } else {
+                        extract_filter_from_filter_node(node, acquired_nodes, draw_ctx)
+                    }
+                }
+
+                _ => {
+                    rsvg_log!(
+                        "element {} will not be filtered since \"{}\" is not a filter",
+                        node_being_filtered_name,
+                        node_id,
+                    );
+                    Err(FilterResolveError::ReferenceToNonFilterElement)
+                }
+            }
+        })
 }
 
 #[derive(Debug, Clone, PartialEq)]
@@ -124,48 +194,6 @@ impl FilterValueList {
     pub fn iter(&self) -> Iter<'_, FilterValue> {
         self.0.iter()
     }
-
-    /// Check that at least one filter URI exists and that all contained
-    /// URIs reference existing <filter> elements.
-    ///
-    /// The `node_name` refers to the node being filtered; it is just
-    /// to log an error in case the filter value list is not
-    /// applicable.
-    pub fn is_applicable(&self, node_name: &str, acquired_nodes: &mut AcquiredNodes<'_>) -> bool {
-        if self.is_empty() {
-            return false;
-        }
-
-        self.iter()
-            .map(|v| match v {
-                FilterValue::Url(v) => v,
-            })
-            .all(|v| match acquired_nodes.acquire(v) {
-                Ok(acquired) => {
-                    let filter_node = acquired.get();
-
-                    match *filter_node.borrow_element() {
-                        Element::Filter(_) => true,
-                        _ => {
-                            rsvg_log!(
-                                "element {} will not be filtered since \"{}\" is not a filter",
-                                node_name,
-                                v,
-                            );
-                            false
-                        }
-                    }
-                }
-                _ => {
-                    rsvg_log!(
-                        "element {} will not be filtered since its filter \"{}\" was not found",
-                        node_name,
-                        v,
-                    );
-                    false
-                }
-            })
-    }
 }
 
 impl Parse for FilterValueList {
@@ -175,10 +203,17 @@ impl Parse for FilterValueList {
         loop {
             let loc = parser.current_source_location();
 
-            let url = parser.expect_url()?;
-            let node_id =
-                NodeId::parse(&url).map_err(|e| loc.new_custom_error(ValueErrorKind::from(e)))?;
-            result.0.push(FilterValue::Url(node_id));
+            let filter_value = if let Ok(func) = parser.try_parse(|p| FilterFunction::parse(p)) {
+                FilterValue::Function(func)
+            } else {
+                let url = parser.expect_url()?;
+                let node_id = NodeId::parse(&url)
+                    .map_err(|e| loc.new_custom_error(ValueErrorKind::from(e)))?;
+
+                FilterValue::Url(node_id)
+            };
+
+            result.0.push(filter_value);
 
             if parser.is_exhausted() {
                 break;
