@@ -8,7 +8,7 @@ use crate::drawing_ctx::DrawingCtx;
 use crate::element::{Draw, Element, ElementResult, SetAttributes};
 use crate::error::*;
 use crate::node::{CascadedValues, Node, NodeBorrow};
-use crate::parsers::{NumberList, NumberListLength, Parse, ParseValue};
+use crate::parsers::{NumberList, Parse, ParseValue};
 use crate::property_defs::ColorInterpolationFilters;
 use crate::rect::IRect;
 use crate::surface_utils::{
@@ -19,7 +19,10 @@ use crate::xml::Attributes;
 
 use super::bounds::BoundsBuilder;
 use super::context::{FilterContext, FilterOutput};
-use super::{FilterEffect, FilterError, Input, Primitive, PrimitiveParams};
+use super::{
+    FilterEffect, FilterError, FilterResolveError, Input, Primitive, PrimitiveParams,
+    ResolvedPrimitive,
+};
 
 /// The `feComponentTransfer` filter primitive.
 #[derive(Default)]
@@ -31,9 +34,9 @@ pub struct FeComponentTransfer {
 /// Resolved `feComponentTransfer` primitive for rendering.
 #[derive(Clone, Default)]
 pub struct ComponentTransfer {
-    in1: Input,
-    functions: Functions,
-    color_interpolation_filters: ColorInterpolationFilters,
+    pub in1: Input,
+    pub functions: Functions,
+    pub color_interpolation_filters: ColorInterpolationFilters,
 }
 
 impl SetAttributes for FeComponentTransfer {
@@ -45,7 +48,7 @@ impl SetAttributes for FeComponentTransfer {
 
 /// Pixel components that can be influenced by `feComponentTransfer`.
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
-enum Channel {
+pub enum Channel {
     R,
     G,
     B,
@@ -54,7 +57,7 @@ enum Channel {
 
 /// Component transfer function types.
 #[derive(Clone, Debug, PartialEq)]
-enum FunctionType {
+pub enum FunctionType {
     Identity,
     Table,
     Discrete,
@@ -86,11 +89,11 @@ struct FunctionParameters {
 }
 
 #[derive(Clone, Debug, Default, PartialEq)]
-struct Functions {
-    r: FeFuncR,
-    g: FeFuncG,
-    b: FeFuncB,
-    a: FeFuncA,
+pub struct Functions {
+    pub r: FeFuncR,
+    pub g: FeFuncG,
+    pub b: FeFuncB,
+    pub a: FeFuncA,
 }
 
 /// The compute function type.
@@ -153,14 +156,14 @@ macro_rules! func_x {
     ($func_name:ident, $channel:expr) => {
         #[derive(Clone, Debug, PartialEq)]
         pub struct $func_name {
-            channel: Channel,
-            function_type: FunctionType,
-            table_values: Vec<f64>,
-            slope: f64,
-            intercept: f64,
-            amplitude: f64,
-            exponent: f64,
-            offset: f64,
+            pub channel: Channel,
+            pub function_type: FunctionType,
+            pub table_values: Vec<f64>,
+            pub slope: f64,
+            pub intercept: f64,
+            pub amplitude: f64,
+            pub exponent: f64,
+            pub offset: f64,
         }
 
         impl Default for $func_name {
@@ -216,9 +219,8 @@ macro_rules! func_x {
                     match attr.expanded() {
                         expanded_name!("", "type") => self.function_type = attr.parse(value)?,
                         expanded_name!("", "tableValues") => {
-                            let NumberList(v) =
-                                NumberList::parse_str(value, NumberListLength::Unbounded)
-                                    .attribute(attr)?;
+                            // #691: Limit list to 256 to mitigate malicious SVGs
+                            let NumberList::<0, 256>(v) = attr.parse(value)?;
                             self.table_values = v;
                         }
                         expanded_name!("", "slope") => self.slope = attr.parse(value)?,
@@ -368,7 +370,11 @@ impl ComponentTransfer {
 }
 
 impl FilterEffect for FeComponentTransfer {
-    fn resolve(&self, node: &Node) -> Result<(Primitive, PrimitiveParams), FilterError> {
+    fn resolve(
+        &self,
+        _acquired_nodes: &mut AcquiredNodes<'_>,
+        node: &Node,
+    ) -> Result<ResolvedPrimitive, FilterResolveError> {
         let cascaded = CascadedValues::new_from_node(node);
         let values = cascaded.get();
 
@@ -376,15 +382,15 @@ impl FilterEffect for FeComponentTransfer {
         params.functions = get_functions(node)?;
         params.color_interpolation_filters = values.color_interpolation_filters();
 
-        Ok((
-            self.base.clone(),
-            PrimitiveParams::ComponentTransfer(params),
-        ))
+        Ok(ResolvedPrimitive {
+            primitive: self.base.clone(),
+            params: PrimitiveParams::ComponentTransfer(params),
+        })
     }
 }
 
 /// Takes a feComponentTransfer and walks its children to produce the feFuncX arguments.
-fn get_functions(node: &Node) -> Result<Functions, FilterError> {
+fn get_functions(node: &Node) -> Result<Functions, FilterResolveError> {
     let func_r_node = get_func_x_node!(node, FeFuncR, Channel::R);
     let func_g_node = get_func_x_node!(node, FeFuncG, Channel::G);
     let func_b_node = get_func_x_node!(node, FeFuncB, Channel::B);
@@ -395,7 +401,7 @@ fn get_functions(node: &Node) -> Result<Functions, FilterError> {
         .filter_map(|x| x.as_ref())
     {
         if node.borrow_element().is_in_error() {
-            return Err(FilterError::ChildNodeInError);
+            return Err(FilterResolveError::ChildNodeInError);
         }
     }
 
