@@ -1,11 +1,12 @@
 use once_cell::sync::OnceCell;
 use std::collections::HashMap;
+use std::rc::Rc;
 
 use crate::bbox::BoundingBox;
 use crate::coord_units::CoordUnits;
 use crate::document::AcquiredNodes;
 use crate::drawing_ctx::DrawingCtx;
-use crate::filter::ResolvedFilter;
+use crate::filter::UserSpaceFilter;
 use crate::paint_server::UserSpacePaintSource;
 use crate::parsers::CustomIdent;
 use crate::property_defs::ColorInterpolationFilters;
@@ -48,9 +49,9 @@ pub enum FilterInput {
 /// The filter rendering context.
 pub struct FilterContext {
     /// Paint source for primitives which have an input value equal to `StrokePaint`.
-    stroke_paint: UserSpacePaintSource,
+    stroke_paint: Rc<UserSpacePaintSource>,
     /// Paint source for primitives which have an input value equal to `FillPaint`.
-    fill_paint: UserSpacePaintSource,
+    fill_paint: Rc<UserSpacePaintSource>,
 
     /// The source graphic surface.
     source_surface: SharedImageSurface,
@@ -99,9 +100,9 @@ pub struct FilterContext {
 impl FilterContext {
     /// Creates a new `FilterContext`.
     pub fn new(
-        filter: &ResolvedFilter,
-        stroke_paint: UserSpacePaintSource,
-        fill_paint: UserSpacePaintSource,
+        filter: &UserSpaceFilter,
+        stroke_paint: Rc<UserSpacePaintSource>,
+        fill_paint: Rc<UserSpacePaintSource>,
         source_surface: &SharedImageSurface,
         draw_transform: Transform,
         node_bbox: BoundingBox,
@@ -257,13 +258,12 @@ impl FilterContext {
 
     /// Stores a filter primitive result into the context.
     #[inline]
-    pub fn store_result(&mut self, result: FilterResult) -> Result<(), FilterError> {
+    pub fn store_result(&mut self, result: FilterResult) {
         if let Some(name) = result.name {
             self.previous_results.insert(name, result.output.clone());
         }
 
         self.last_result = Some(result.output);
-        Ok(())
     }
 
     /// Returns the paffine matrix.
