@@ -19,7 +19,10 @@ use crate::xml::Attributes;
 
 use super::bounds::BoundsBuilder;
 use super::context::{FilterContext, FilterOutput};
-use super::{FilterEffect, FilterError, Input, Primitive, PrimitiveParams};
+use super::{
+    FilterEffect, FilterError, FilterResolveError, Input, Primitive, PrimitiveParams,
+    ResolvedPrimitive,
+};
 
 /// The maximum gaussian blur kernel size.
 ///
@@ -36,9 +39,9 @@ pub struct FeGaussianBlur {
 /// Resolved `feGaussianBlur` primitive for rendering.
 #[derive(Default, Clone)]
 pub struct GaussianBlur {
-    in1: Input,
-    std_deviation: (f64, f64),
-    color_interpolation_filters: ColorInterpolationFilters,
+    pub in1: Input,
+    pub std_deviation: (f64, f64),
+    pub color_interpolation_filters: ColorInterpolationFilters,
 }
 
 impl SetAttributes for FeGaussianBlur {
@@ -244,13 +247,20 @@ impl GaussianBlur {
 }
 
 impl FilterEffect for FeGaussianBlur {
-    fn resolve(&self, node: &Node) -> Result<(Primitive, PrimitiveParams), FilterError> {
+    fn resolve(
+        &self,
+        _acquired_nodes: &mut AcquiredNodes<'_>,
+        node: &Node,
+    ) -> Result<ResolvedPrimitive, FilterResolveError> {
         let cascaded = CascadedValues::new_from_node(node);
         let values = cascaded.get();
 
         let mut params = self.params.clone();
         params.color_interpolation_filters = values.color_interpolation_filters();
 
-        Ok((self.base.clone(), PrimitiveParams::GaussianBlur(params)))
+        Ok(ResolvedPrimitive {
+            primitive: self.base.clone(),
+            params: PrimitiveParams::GaussianBlur(params),
+        })
     }
 }
