@@ -2,12 +2,14 @@
 //!
 //! This module provides the primitives on which the public APIs are implemented.
 
+use crate::accept_language::UserLanguage;
 use crate::bbox::BoundingBox;
 use crate::css::{Origin, Stylesheet};
 use crate::document::{AcquiredNodes, Document, NodeId};
 use crate::dpi::Dpi;
 use crate::drawing_ctx::{draw_tree, DrawingMode, ViewParams};
 use crate::error::{DefsLookupErrorKind, LoadingError, RenderingError};
+use crate::length::*;
 use crate::node::{CascadedValues, Node, NodeBorrow};
 use crate::rect::Rect;
 use crate::structure::IntrinsicDimensions;
@@ -130,12 +132,14 @@ impl Handle {
             return None;
         }
 
-        let params = ViewParams::new(dpi, 0.0, 0.0);
+        let view_params = ViewParams::new(dpi, 0.0, 0.0);
         let root = self.document.root();
         let cascaded = CascadedValues::new_from_node(&root);
         let values = cascaded.get();
 
-        Some((w.normalize(values, &params), h.normalize(values, &params)))
+        let params = NormalizeParams::new(&values, &view_params);
+
+        Some((w.to_user(&params), h.to_user(&params)))
     }
 
     fn get_node_or_root(&self, id: Option<&str>) -> Result<Node, RenderingError> {
@@ -150,6 +154,7 @@ impl Handle {
         &self,
         node: Node,
         viewport: Rect,
+        user_language: &UserLanguage,
         dpi: Dpi,
         is_testing: bool,
     ) -> Result<(Rect, Rect), RenderingError> {
@@ -162,6 +167,7 @@ impl Handle {
             DrawingMode::LimitToStack { node, root },
             &cr,
             viewport,
+            user_language,
             dpi,
             true,
             is_testing,
@@ -178,13 +184,15 @@ impl Handle {
         &self,
         id: Option<&str>,
         viewport: &cairo::Rectangle,
+        user_language: &UserLanguage,
         dpi: Dpi,
         is_testing: bool,
     ) -> Result<(cairo::Rectangle, cairo::Rectangle), RenderingError> {
         let viewport = Rect::from(*viewport);
         let node = self.get_node_or_root(id)?;
 
-        let (ink_rect, logical_rect) = self.geometry_for_layer(node, viewport, dpi, is_testing)?;
+        let (ink_rect, logical_rect) =
+            self.geometry_for_layer(node, viewport, user_language, dpi, is_testing)?;
 
         Ok((
             cairo::Rectangle::from(ink_rect),
@@ -223,10 +231,11 @@ impl Handle {
         &self,
         cr: &cairo::Context,
         viewport: &cairo::Rectangle,
+        user_language: &UserLanguage,
         dpi: Dpi,
         is_testing: bool,
     ) -> Result<(), RenderingError> {
-        self.render_layer(cr, None, viewport, dpi, is_testing)
+        self.render_layer(cr, None, viewport, user_language, dpi, is_testing)
     }
 
     pub fn render_layer(
@@ -234,6 +243,7 @@ impl Handle {
         cr: &cairo::Context,
         id: Option<&str>,
         viewport: &cairo::Rectangle,
+        user_language: &UserLanguage,
         dpi: Dpi,
         is_testing: bool,
     ) -> Result<(), RenderingError> {
@@ -250,6 +260,7 @@ impl Handle {
             DrawingMode::LimitToStack { node, root },
             cr,
             viewport,
+            user_language,
             dpi,
             false,
             is_testing,
@@ -264,6 +275,7 @@ impl Handle {
     fn get_bbox_for_element(
         &self,
         node: &Node,
+        user_language: &UserLanguage,
         dpi: Dpi,
         is_testing: bool,
     ) -> Result<BoundingBox, RenderingError> {
@@ -276,6 +288,7 @@ impl Handle {
             DrawingMode::OnlyNode(node),
             &cr,
             unit_rectangle(),
+            user_language,
             dpi,
             true,
             is_testing,
@@ -287,12 +300,13 @@ impl Handle {
     pub fn get_geometry_for_element(
         &self,
         id: Option<&str>,
+        user_language: &UserLanguage,
         dpi: Dpi,
         is_testing: bool,
     ) -> Result<(cairo::Rectangle, cairo::Rectangle), RenderingError> {
         let node = self.get_node_or_root(id)?;
 
-        let bbox = self.get_bbox_for_element(&node, dpi, is_testing)?;
+        let bbox = self.get_bbox_for_element(&node, user_language, dpi, is_testing)?;
 
         let ink_rect = bbox.ink_rect.unwrap_or_default();
         let logical_rect = bbox.rect.unwrap_or_default();
@@ -311,6 +325,7 @@ impl Handle {
         cr: &cairo::Context,
         id: Option<&str>,
         element_viewport: &cairo::Rectangle,
+        user_language: &UserLanguage,
         dpi: Dpi,
         is_testing: bool,
     ) -> Result<(), RenderingError> {
@@ -318,7 +333,7 @@ impl Handle {
 
         let node = self.get_node_or_root(id)?;
 
-        let bbox = self.get_bbox_for_element(&node, dpi, is_testing)?;
+        let bbox = self.get_bbox_for_element(&node, user_language, dpi, is_testing)?;
 
         if bbox.ink_rect.is_none() || bbox.rect.is_none() {
             // Nothing to draw
@@ -346,6 +361,7 @@ impl Handle {
             DrawingMode::OnlyNode(node),
             &cr,
             unit_rectangle(),
+            user_language,
             dpi,
             false,
             is_testing,
