@@ -63,6 +63,7 @@ use crate::drawing_ctx::ViewParams;
 use crate::error::*;
 use crate::parsers::{finite_f32, Parse};
 use crate::properties::ComputedValues;
+use crate::viewbox::ViewBox;
 
 /// Units for length values.
 // This needs to be kept in sync with `rsvg.h:RsvgUnit`.
@@ -327,6 +328,25 @@ impl<N: Normalize, V: Validate> Parse for CssLength<N, V> {
     }
 }
 
+/// Parameters to normalize `Length` values to user-space distances.
+pub struct NormalizeParams {
+    vbox: ViewBox,
+    font_size: f64,
+    dpi: Dpi,
+}
+
+impl NormalizeParams {
+    /// Extracts the information needed to normalize `Length` values from a set of
+    /// `ComputedValues` and the viewport size in `ViewParams`.
+    pub fn new(values: &ComputedValues, params: &ViewParams) -> NormalizeParams {
+        NormalizeParams {
+            vbox: params.vbox,
+            font_size: font_size_from_values(values, params.dpi),
+            dpi: params.dpi,
+        }
+    }
+}
+
 impl<N: Normalize, V: Validate> CssLength<N, V> {
     /// Creates a CssLength.
     ///
@@ -351,13 +371,15 @@ impl<N: Normalize, V: Validate> CssLength<N, V> {
         }
     }
 
-    /// Normalizes a specified length into a used value.
+    /// Convert a Length with units into user-space coordinates.
     ///
     /// Lengths may come with non-pixel units, and when rendering, they need to be normalized
     /// to pixels based on the current viewport (e.g. for lengths with percent units), and
     /// based on the current element's set of `ComputedValues` (e.g. for lengths with `Em`
     /// units that need to be resolved against the current font size).
-    pub fn normalize(&self, values: &ComputedValues, params: &ViewParams) -> f64 {
+    ///
+    /// Those parameters can be obtained with `NormalizeParams::new()`.
+    pub fn to_user(&self, params: &NormalizeParams) -> f64 {
         match self.unit {
             LengthUnit::Px => self.length,
 
@@ -365,9 +387,9 @@ impl<N: Normalize, V: Validate> CssLength<N, V> {
                 self.length * <N as Normalize>::normalize(params.vbox.width(), params.vbox.height())
             }
 
-            LengthUnit::Em => self.length * font_size_from_values(values, params.dpi),
+            LengthUnit::Em => self.length * params.font_size,
 
-            LengthUnit::Ex => self.length * font_size_from_values(values, params.dpi) / 2.0,
+            LengthUnit::Ex => self.length * params.font_size / 2.0,
 
             LengthUnit::In => self.length * <N as Normalize>::normalize(params.dpi.x, params.dpi.y),
 
@@ -526,81 +548,81 @@ mod tests {
 
     #[test]
     fn normalize_default_works() {
-        let params = ViewParams::new(Dpi::new(40.0, 40.0), 100.0, 100.0);
-
+        let view_params = ViewParams::new(Dpi::new(40.0, 40.0), 100.0, 100.0);
         let values = ComputedValues::default();
+        let params = NormalizeParams::new(&values, &view_params);
 
         assert_approx_eq_cairo!(
-            Length::<Both>::new(10.0, LengthUnit::Px).normalize(&values, &params),
+            Length::<Both>::new(10.0, LengthUnit::Px).to_user(&params),
             10.0
         );
     }
 
     #[test]
     fn normalize_absolute_units_works() {
-        let params = ViewParams::new(Dpi::new(40.0, 50.0), 100.0, 100.0);
-
+        let view_params = ViewParams::new(Dpi::new(40.0, 50.0), 100.0, 100.0);
         let values = ComputedValues::default();
+        let params = NormalizeParams::new(&values, &view_params);
 
         assert_approx_eq_cairo!(
-            Length::<Horizontal>::new(10.0, LengthUnit::In).normalize(&values, &params),
+            Length::<Horizontal>::new(10.0, LengthUnit::In).to_user(&params),
             400.0
         );
         assert_approx_eq_cairo!(
-            Length::<Vertical>::new(10.0, LengthUnit::In).normalize(&values, &params),
+            Length::<Vertical>::new(10.0, LengthUnit::In).to_user(&params),
             500.0
         );
 
         assert_approx_eq_cairo!(
-            Length::<Horizontal>::new(10.0, LengthUnit::Cm).normalize(&values, &params),
+            Length::<Horizontal>::new(10.0, LengthUnit::Cm).to_user(&params),
             400.0 / CM_PER_INCH
         );
         assert_approx_eq_cairo!(
-            Length::<Horizontal>::new(10.0, LengthUnit::Mm).normalize(&values, &params),
+            Length::<Horizontal>::new(10.0, LengthUnit::Mm).to_user(&params),
             400.0 / MM_PER_INCH
         );
         assert_approx_eq_cairo!(
-            Length::<Horizontal>::new(10.0, LengthUnit::Pt).normalize(&values, &params),
+            Length::<Horizontal>::new(10.0, LengthUnit::Pt).to_user(&params),
             400.0 / POINTS_PER_INCH
         );
         assert_approx_eq_cairo!(
-            Length::<Horizontal>::new(10.0, LengthUnit::Pc).normalize(&values, &params),
+            Length::<Horizontal>::new(10.0, LengthUnit::Pc).to_user(&params),
             400.0 / PICA_PER_INCH
         );
     }
 
     #[test]
     fn normalize_percent_works() {
-        let params = ViewParams::new(Dpi::new(40.0, 40.0), 100.0, 200.0);
-
+        let view_params = ViewParams::new(Dpi::new(40.0, 40.0), 100.0, 200.0);
         let values = ComputedValues::default();
+        let params = NormalizeParams::new(&values, &view_params);
 
         assert_approx_eq_cairo!(
-            Length::<Horizontal>::new(0.05, LengthUnit::Percent).normalize(&values, &params),
+            Length::<Horizontal>::new(0.05, LengthUnit::Percent).to_user(&params),
             5.0
         );
         assert_approx_eq_cairo!(
-            Length::<Vertical>::new(0.05, LengthUnit::Percent).normalize(&values, &params),
+            Length::<Vertical>::new(0.05, LengthUnit::Percent).to_user(&params),
             10.0
         );
     }
 
     #[test]
     fn normalize_font_em_ex_works() {
-        let params = ViewParams::new(Dpi::new(40.0, 40.0), 100.0, 200.0);
-
+        let view_params = ViewParams::new(Dpi::new(40.0, 40.0), 100.0, 200.0);
         let values = ComputedValues::default();
+        let params = NormalizeParams::new(&values, &view_params);
 
         // These correspond to the default size for the font-size
         // property and the way we compute Em/Ex from that.
 
         assert_approx_eq_cairo!(
-            Length::<Vertical>::new(1.0, LengthUnit::Em).normalize(&values, &params),
+            Length::<Vertical>::new(1.0, LengthUnit::Em).to_user(&params),
             12.0
         );
 
         assert_approx_eq_cairo!(
-            Length::<Vertical>::new(1.0, LengthUnit::Ex).normalize(&values, &params),
+            Length::<Vertical>::new(1.0, LengthUnit::Ex).to_user(&params),
             6.0
         );
     }
