@@ -180,6 +180,7 @@ mod property_macros;
 #[macro_use]
 mod util;
 
+mod accept_language;
 mod angle;
 mod api;
 mod aspect_ratio;
@@ -195,6 +196,7 @@ mod drawing_ctx;
 mod element;
 mod error;
 mod filter;
+mod filter_func;
 pub mod filters;
 mod font_props;
 mod gradient;
@@ -203,6 +205,7 @@ mod href;
 mod image;
 mod io;
 mod iri;
+mod layout;
 mod length;
 mod limits;
 mod marker;
