@@ -109,11 +109,6 @@ impl EllipticalArc {
             to: (x2, y2),
         } = self;
 
-        // If the end points are identical, omit the arc segment entirely.
-        if x1.approx_eq_cairo(x2) && y1.approx_eq_cairo(y2) {
-            return ArcParameterization::Omit;
-        }
-
         // Ensure radii are non-zero.
         // Otherwise this arc is treated as a line segment joining the end points.
         //
