@@ -549,7 +549,11 @@ impl SpecifiedValues {
 
     pub fn to_computed_values(&self, computed: &mut ComputedValues) {
         macro_rules! compute {
-            ($name:ident, $field:ident) => {
+            ($name:ident, $field:ident) => {{
+                // This extra block --------^
+                // is so that prop_val will be dropped within the macro invocation;
+                // otherwise all the temporary values cause this function to use
+                // an unreasonably large amount of stack space.
                 let prop_val = self.get_property(PropertyId::$name);
                 if let ParsedProperty::$name(s) = prop_val {
                     computed.set_value(ComputedValue::$name(
@@ -558,7 +562,7 @@ impl SpecifiedValues {
                 } else {
                     unreachable!();
                 }
-            };
+            }};
         }
 
         // First, compute font_size.  It needs to be done before everything
@@ -633,11 +637,7 @@ impl SpecifiedValues {
         }
     }
 
-    fn parse_one_presentation_attribute(
-        &mut self,
-        attr: QualName,
-        value: &str,
-    ) -> Result<(), ElementError> {
+    fn parse_one_presentation_attribute(&mut self, attr: QualName, value: &str) {
         let mut input = ParserInput::new(value);
         let mut parser = Parser::new(&mut input);
 
@@ -718,8 +718,6 @@ impl SpecifiedValues {
                 );
             }
         }
-
-        Ok(())
     }
 
     pub fn parse_presentation_attributes(
@@ -746,7 +744,7 @@ impl SpecifiedValues {
                     )));
                 }
 
-                _ => self.parse_one_presentation_attribute(attr, value)?,
+                _ => self.parse_one_presentation_attribute(attr, value),
             }
         }
 
@@ -815,7 +813,7 @@ where
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::iri::IRI;
+    use crate::iri::Iri;
     use crate::length::*;
 
     #[test]
@@ -867,7 +865,7 @@ mod tests {
     #[test]
     fn expands_marker_shorthand() {
         let mut specified = SpecifiedValues::default();
-        let iri = IRI::parse_str("url(#foo)").unwrap();
+        let iri = Iri::parse_str("url(#foo)").unwrap();
 
         let marker = Marker(iri.clone());
         specified.set_parsed_property(&ParsedProperty::Marker(SpecifiedValue::Specified(marker)));
@@ -883,8 +881,8 @@ mod tests {
     #[test]
     fn replaces_marker_shorthand() {
         let mut specified = SpecifiedValues::default();
-        let iri1 = IRI::parse_str("url(#foo)").unwrap();
-        let iri2 = IRI::None;
+        let iri1 = Iri::parse_str("url(#foo)").unwrap();
+        let iri2 = Iri::None;
 
         let marker1 = Marker(iri1.clone());
         specified.set_parsed_property(&ParsedProperty::Marker(SpecifiedValue::Specified(marker1)));
@@ -940,4 +938,13 @@ mod tests {
 
         assert_eq!(computed.opacity(), half_opacity.clone());
     }
+
+    #[test]
+    fn empty_style_attribute_parses_ok() {
+        let mut specified = SpecifiedValues::default();
+
+        assert!(specified
+            .parse_style_declarations("", Origin::Author, &mut HashSet::new())
+            .is_ok())
+    }
 }
